import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

from . import utils

class ReportCycleItem(Model):
    _name="clinic.report.cycle.item"
    _string="Report Cycle Item"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        "cycle_id": fields.Many2One("clinic.cycle","Cycle"),
        "ptype_id": fields.Many2One("clinic.patient.type","Patient Type"),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
        'department_id': fields.Many2One("clinic.department","Department"),
    }

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date=defaults.get('date',time.strftime("%Y-%m-%d"))
        year,month=time.strftime("%Y-%m").split("-")
        date_from=defaults.get('date_from','%s-%s-01'%(year,month))
        date_to=defaults.get('date_to','%s-%s-01'%(year,month))
        branch_id=defaults.get('branch_id')
        ptype_id=defaults.get('ptype_id')
        department_id=defaults.get('department_id')
        res=get_model('select.company').get_select()
        if res:
            if not branch_id:
                branch_id=res['branch_id']
            if not department_id:
                if res.get("department_ids"):
                    department_id=res['department_ids'][0]
                else:
                    department_id=res['department_id']
        res={
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'branch_id': branch_id,
            'department_id': department_id,
        }
        print('res', res)
        return res
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model('company').browse(company_id)
        defaults=self.default_get(context=context)
        date_from=defaults.get("date_from")
        date_to=defaults.get("date_to")
        branch_id=defaults.get("branch_id")
        department_id=defaults.get("department_id")
        ptype_id=defaults.get("ptype_id")
        month=date_from.split("-")[1]
        cycle_id=None
        if ids:
            obj=self.browse(ids)[0]
            month=obj.date_from.split("-")[1]
            date_from=obj.date_from
            date_to=obj.date_to
            branch_id=obj.branch_id.id
            department_id=obj.department_id.id
            cycle_id=obj.cycle_id.id
            ptype_id=obj.ptype_id.id
        # new patient of this month
        dom=[]
        dom.append(['date','>=',date_from])
        dom.append(['date','<=',date_to])
        dom.append(['state','=','validated'])
        if cycle_id:
            dom.append(['cycle_id','=',cycle_id])
        if branch_id:
            dom.append(['branch_id','=',branch_id])
        if department_id:
            dom.append(['department_id','=',department_id])
        lines=[]
        cycles={}
        for citem in get_model('clinic.cycle.item').search_browse(dom,order="date"):
            cycle=citem.cycle_id
            if cycle.id not in cycles.keys():
                cycles[cycle.id]=[]
                for line in citem.lines:
                    nurse=line.nurse_id
                    cycles[cycle.id].append(nurse.name)
            for hdcase in citem.hd_cases:
                patient=hdcase.patient_id
                ptype=patient.type_id
                if ptype_id and ptype_id!=ptype.id:
                    continue
                doctor=hdcase.doctor_id
                cycle=hdcase.cycle_id
                dpt=hdcase.department_id
                dlz_use=hdcase.dlz_use or 0
                dlz_drop=False
                if dlz_use==hdcase.dlz_max:
                    dlz_use="%sทิ้ง"%dlz_use
                    dlz_drop=True
                cancel=False
                if hdcase.state=='cancelled':
                    cancel=True
                lines.append({
                   'dlz_drop': dlz_drop,
                   'cancel': cancel,
                   'pname': patient.name or '',
                   'pid': patient.id or '',
                   'hn': patient.hn_no,
                   'did': doctor.id,
                   'dname': doctor.name or "",
                   'date': hdcase.date,
                   'epo': hdcase.epo,
                   'mdc': hdcase.mdc,
                   'mdc_name': hdcase.mdc_name,
                   'fee': abs(hdcase.fee),
                   'dlz_name': hdcase.dlz_name,
                   'dlz_use': dlz_use,
                   'dlz_id': hdcase.dlz_id,
                   'cseq': cycle.sequence or 0,
                   'cid': cycle.id,
                   'hdcase_id': hdcase.id,
                   'note': hdcase.note or "",
                   'cname': cycle.name or '',
                   'ctid': citem.id or '',
                   'hct': hdcase.hct or 0,
                   'tname': ptype.name or '',
                   'tid': ptype.id,
                   'dpt_id': dpt.id,
                   'dpt_name': dpt.name or "",
                })
        month_str=utils.MONTHS['th_TH'][int(month)]
        company_name=company.name or ""
        if department_id:
            department=get_model("clinic.department").browse(department_id)
            company_name+=" ("+department.name+")"
        elif branch_id:
            branch=get_model("clinic.branch").browse(branch_id)
            company_name+=" ("+branch.name+")"
        no=1
        nlines=[]
        index=0
        old=[]
        total_fee=0
        total_mdc=0
        for line in sorted(lines,key=lambda x:(x['date'],x['cseq'])):
            total_fee+=line.get("fee",0)
            total_mdc+=line.get("mdc",0)
            cid=line['cid']
            if not cid in old:
                old.append(cid)
                index=0
            else:
                index+=1
            cres=cycles[cid]
            line['nurse']=''
            if index < len(cres):
                line['nurse']=cres[index]
            line['no']=no
            nlines.append(line)
            no+=1
        data={
            'company_name': company_name or "",
            'lines': nlines,
            'month': month_str,
            'date_from': date_from,
            'date_to': date_to,
            'total_fee': total_fee,
            'total_mdc': total_mdc,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_type(self,context={}):
        data=context['data']
        data['department_id']=None
        return data

    def onchange_datefrom(self,context={}):
        data=context['data']
        data['date_to']=data['date_from']
        return data

ReportCycleItem.register()
