import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

class ReportLaborCostDetail(Model):
    _name="clinic.report.labor.cost.detail"
    _string="Report Labor Cost Detail"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'department_id': fields.Many2One("clinic.department","Department"),
        'staff_id': fields.Many2One("clinic.staff","Staff"),
        "type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type"),
        'department_id': fields.Many2One("clinic.department","Department"),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
    }

    def _get_date_from(self,context={}):
        year,month=time.strftime("%Y-%m").split("-")
        return '%s-%s-01'%(year,month)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        return "%s-%s-%s"%(year,month,total_day)

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date_from=defaults.get("date_from")
        if not date_from:
            date_from=self._get_date_from()
        date_to=defaults.get("date_to")
        if not date_to:
            date_to=self._get_date_to()
        staff_type=defaults.get("staff_type","doctor")
        staff_id=int(defaults.get('staff_id', "0"))
        department_id=int(defaults.get('department_id', "0"))
        res={
            'date': time.strftime("%Y-%m-%d"),
            'date_from': date_from,
            'date_to': date_to,
            'type': staff_type,
            'staff_id': staff_id and staff_id or None,
            'department_id': department_id and department_id or None,
        }
        return res

    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        comp=get_model("company").browse(company_id)
        defaults=self.default_get(context=context)
        date_from=defaults.get("date_from")
        date_to=defaults.get("date_to")
        staff_type=defaults.get("type")
        staff_id=defaults.get("staff_id")
        department_id=defaults.get("department_id")
        dom=[]
        if ids:
            obj=self.browse(ids)[0]
            date_from=obj.date_from
            date_to=obj.date_to
            staff_id=obj.staff_id.id
            staff_type=obj.type
            department_id=obj.department_id.id
        dom.append(['date','>=',date_from])
        dom.append(['date','<=',date_to])
        if staff_id:
            dom.append(['staff_id','=',staff_id])
        if staff_type:
            dom.append(['type','=',staff_type])
        if department_id:
            dom.append(['labor_cost_id.cycle_item_id.department_id','=',department_id])

        def replace_quote(dom=""):
            return dom.replace("'","\"")
        #prevent to load more data
        if not staff_id:
            return {}
        dates={}
        for line in get_model("clinic.labor.cost.line").search_browse(dom):
            lcost=line.labor_cost_id
            citem=lcost.cycle_item_id
            date=citem.date
            if not date:
                continue
            dpt=citem.department_id
            amt=line.amount or 0
            staff=line.staff_id
            qty=0
            if staff.type=='doctor':
                for hdcase in citem.hd_cases:
                    doctor_id=hdcase.doctor_id.id
                    if staff.id==doctor_id:
                        qty+=1
            elif staff.type=='nurse':
                qty=len(citem.hd_cases) or 0 # qty of patient
            else:
                pass
            if not dates.get(date):
                dates[date]={
                    'labor_cost_id': lcost.id,
                    'staff_id': staff.id,
                    'staff_type': staff.type,
                    dpt.name: {
                        'amt': 0,
                        'qty': 0,
                        'dom': [['staff_id','=',staff.id],['date','>=', date],['date','<=',date], ['department_id','=',dpt.id]],
                        },
                    }
            if not dates[date].get(dpt.name):
                dates[date].update({
                        'labor_cost_id': lcost.id,
                        'staff_id': staff.id,
                        'staff_type': staff.type,
                        dpt.name: {
                            'amt': 0,
                            'qty': 0,
                            'dom': [['staff_id','=',staff.id],['date','>=', date],['date','<=',date], ['department_id','=',dpt.id]],
                        }})
            dates[date][dpt.name]['amt']+=amt
            dates[date][dpt.name]['qty']+=qty
        lines=[]
        dpts=get_model("clinic.department").search_read([],['name'])
        department_name=""
        if department_id:
            department_name=get_model("clinic.department").browse(department_id).name or ""
        # link to sub detail
        show_count=0
        for dpt in dpts:
            show_link=True
            if dpt['name']!=department_name and department_name!='':
                show_link=False
                show_count+=1
            dpt.update({
                'staff_id': staff_id,
                'staff_type': staff_type,
                'date_from': date_from,
                'date_to': date_to,
                'show_link': show_link,
            })
        dpts=sorted(dpts, key=lambda b: b['name'])
        no=1
        dlines=sorted(dates.keys()) #sort by staff name
        for kdate in dlines:
            vals=dates[kdate]
            lvals={
                'no': no,
                'date': kdate,
                'staff_type': vals.get("staff_type"),
                'staff_id': vals.get("staff_id"),
                'labor_cost_id': vals.get("labor_cost_id")
            }
            total_qty, total_amt=0, 0
            lvals['sub_lines']=[]
            for dpt in dpts:
                amt,qty=0,0
                dname=dpt['name'] or ''
                dpt_vals=vals.get(dname)
                url_name=None
                if dpt_vals:
                    amt=dpt_vals.get("amt",0)
                    qty=dpt_vals.get("qty",0)
                    dom=dpt_vals.get('dom',[])
                    if dom:
                        url_name='#name=clinic_labor_cost_item&mode=list&search_domain=%s&tab_no=0'%replace_quote('%s'%dom),
                lvals['sub_lines'].append({
                        'qty': qty, 
                        'amt': amt, 
                        'dpt_id': dpt['id'],
                        'url_name': url_name,
                    })
                total_amt+=amt
                total_qty+=qty
            lvals['total_qty']=total_qty # total show as right hand side
            lvals['total_amt']=total_amt
            lines.append(lvals)
            no+=1
        # summary as footer
        dpt_lines=[{'qty': 0, 'amt': 0} for dpt in dpts]
        for line in lines:
            i=0
            for sub_line in line['sub_lines']:
                dpt_lines[i]['qty']+=sub_line['qty'] or 0
                dpt_lines[i]['amt']+=sub_line['amt'] or 0
                i+=1

        ## summary as footer (total)
        total_qty, total_amt=0, 0
        for tline in dpt_lines:
            total_qty+=tline['qty'] or 0
            total_amt+=tline['amt'] or 0
        dpt_lines.append({'amt': total_amt, 'qty': total_qty})
        
        for dpt in dpts:
            dpt.update({
                'qty_text': 'คนไข้',
                'amt_text': 'จำนวนเงิน',
            })
        comp_span=(len(dpts)*2)+1
        staff_name=''
        if staff_id:
            staff=get_model("clinic.staff").browse(staff_id)
            staff_name=staff.name or '' 
        data={
            'staff_name': staff_name,
            'staff_id': staff_id,
            'staff_type': staff_type,
            'department_id': "0", #XXX
            'date_from': date_from,
            'date_to': date_to,
            'dpts': dpts,
            'comp_name': comp.name or 0,
            'comp_span': comp_span, #qty, amt
            'lines': lines,
            'dpt_lines': dpt_lines,
            'show_all': show_count <=1 and True or False,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_type(self,context={}):
        data=context['data']
        data['staff_id']=None
        return data

ReportLaborCostDetail.register()
